﻿/*========================================================================================
(c) Dmitriy N. Sakharov
TableStyle v2.0
Скрипт для приведения к заданному виду таблиц в публикации.

Может работать с таблицами в выделении, стори, документе.
Позволяет настраивать:
- отступы до и после таблицы;
- отступы в ячейках сверху, снизу, слева, справа;
- тип линий;
- цвет линий;
- оттенок цвета линий;
- толщину линий;
- цвет заливки ячеек;
- оттенок цвета заливки ячеек;
- тип вертикального выравнивания в ячейках;
- стиль текста в таблице;
- высоту строк,
а также удалять лишние пробелы и знаки конца абзаца в начале и конце ячеек 
(при этом скрипт может работать довольно продолжительное время).

ИЗМЕНЕНИЯ в версии 2.0:
- добавлена ProgressBar;
- форматирование таблиц происходит с использованием стилей таблиц и ячеек;
- увеличена скорость работы скрипта из-за изменения алгоритма обработки текста;
- исправлена ошибка подсчета времени работы.

ИЗМЕНЕНИЯ в версии 1.5:
- исправлена ошибка работы скрипта при выделении одной таблицы;
- добавлена возможность выбора единиц измерения (мм или пт);
- добавлена функция запоминания настроек.

Отзывы и замечания по работе скрипта направляйте на sakhar@mail.ru

ЕСЛИ СКРИПТ ПОМОГАЕТ ВАМ В РАБОТЕ, ОБЛЕГЧАЯ ВАШ ТРУД,
И ВЫ ХОТИТЕ ОТБЛАГОДАРИТЬ АВТОРА - ОКАЖИТЕ СПОНСОРСКУЮ ПОМОЩЬ В ЛЮБОМ РАЗМЕРЕ!!!
Узнать, как это сделать, можно по e-mail: sakhar@mail.ru.
=========================================================================================*/

//app.scriptPreferences.version = 4;
with(app){
	// Проверка открытых документов
	if(app.documents.length==0){
		alert("Нет открытых документов!");
		exit();
	}				
	// Проверка наличия	файла настроек
	var myIniFile = new File (activeScript.path + "/TableStyle.ini");
	if (!myIniFile.exists) {
		var myUnits = " mm";
		var mySpaceBeforeSave = mySpaceAfterSave = 1;
		var myStyleSave = myColorSave = myFillSave = myParagraphStyleSave = "";
		var myTintSave = myFillTintSave = 100;
		var myStrokeSave = 0.5;
		var myTopSave = myBottomSave = myLeftSave = myRightSave = 1;
		var myAlignSave = 0;
		var myEnd = myControl = myAtLeast = true;
	}
	else {readFile(myIniFile);}	
	//Переменные по умолчанию
	var myControlSave = myEndSave = myAtLeastSave = "";
	var myDoc = activeDocument;
	var myStrokeStyles = myDoc.strokeStyles;
	var mySwatches = myDoc.swatches;
	var myParaStylesList = new Array();
	var myStrokeStylesList = new Array();
	var mySwatchesList = new Array();
	var mySizeList = new Array("0","0.1","0.25","0.5","0.75","1","2","3","4","5","6","7","8","9","10");
	var myPercentList = new Array("5","10","15","20","25","30","35","40","45","50","55","60","65","70","75","80","85","90","95","100");
	var myAlignList = new Array("Top","Center","Bottom","Justify");
	var myStyleId = myColorId = myFillId = myParagraphStyleId = 0;	
	// Список стилей в публикации
	var myParaStylesList = new Array();
	for (myCounter = 0; myCounter < activeDocument.paragraphStyles.length; myCounter++) {
		myParaStylesList[myCounter] = activeDocument.paragraphStyles.item(myCounter).name;
		if (myParaStylesList[myCounter] == myParagraphStyleSave) {
			myParagraphStyleId = myCounter;
		}
	}
	// Список стилей линии
	for (var myCounter = 0; myCounter < myStrokeStyles.length; myCounter++) {
		myStrokeStylesList[myCounter] = myStrokeStyles[myCounter].name;
		if (myStrokeStylesList[myCounter] == myStyleSave) {
			myStyleId = myCounter;
		}
	}
	// Список цветов в публикации
	for (var myCounter = 0; myCounter < mySwatches.length; myCounter++) {
		mySwatchesList[myCounter] = mySwatches[myCounter].name;
		if (mySwatchesList[myCounter] == myColorSave) {
			myColorId = myCounter;
		}
		if (mySwatchesList[myCounter] == myFillSave) {
			myFillId = myCounter;
		}
	}
	// Проверка выделения
	if (selection.length !=0 && selection[0].constructor.name != "InsertionPoint" && selection[0].constructor.name != "TextFrame") {
		var myCheck1 = true;
		var myCheck2 = false;
	}
	else {
		var myCheck1 = false;
		var myCheck2 = true;
	}		
	// Диалог
	var myDialog = dialogs.add({name:"TableStyle v2.0"});
	with (myDialog.dialogColumns.add()) {
		with (dialogRows.add().dialogColumns.add()) {
			with (dialogRows.add()) {
				staticTexts.add({staticLabel:"Обрабатывать таблицы в: "});
				var myRadioButton = radiobuttonGroups.add();
				with(myRadioButton){
					var mySelectRadioButton = radiobuttonControls.add({staticLabel:"Selection", checkedState:myCheck1});
					var myDocRadioButton = radiobuttonControls.add({staticLabel:"Document"});
					var myStoryRadioButton = radiobuttonControls.add({staticLabel:"Story", checkedState:myCheck2});
				}				
			}
			with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Единицы измерения:", minWidth:50});
					var myRadioButton1 = radiobuttonGroups.add();
					with(myRadioButton1){
						var myMmRadioButton = radiobuttonControls.add({staticLabel:"mm"});
						var myPtRadioButton = radiobuttonControls.add({staticLabel:"pt"});
						if (myUnits == " mm") {
							myMmRadioButton.checkedState = true;
						}
						else {myPtRadioButton.checkedState = true;}
					}					
			}			
			with (dialogRows.add().dialogColumns.add()) {
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Space:", minWidth:55});
					staticTexts.add({staticLabel:"before:", minWidth:50});
					var mySelectedSpaceBefore = realEditboxes.add({editValue:mySpaceBeforeSave, smallNudge:0.25, largeNudge:1, maximumValue:100, minWidth:30});
					staticTexts.add({staticLabel:"after:", minWidth:50});
					var mySelectedSpaceAfter = realEditboxes.add({editValue:mySpaceAfterSave, smallNudge:0.25, largeNudge:1, maximumValue:100, minWidth:30});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Cell inset:", minWidth:55});
					staticTexts.add({staticLabel:"top:", minWidth:50});
					var mySelectedTop = realEditboxes.add({editValue:myTopSave, smallNudge:0.25, largeNudge:1, maximumValue:100, minWidth:30});
					staticTexts.add({staticLabel:"bottom:", minWidth:50});
					var mySelectedBottom = realEditboxes.add({editValue:myBottomSave, smallNudge:0.25, largeNudge:1, maximumValue:100, minWidth:30});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"left:", minWidth:110});
					var mySelectedLeft = realEditboxes.add({editValue:myLeftSave, smallNudge:0.25, largeNudge:1, maximumValue:100, minWidth:30});
					staticTexts.add({staticLabel:"right:", minWidth:50});
					var mySelectedRight = realEditboxes.add({editValue:myRightSave, smallNudge:0.25, largeNudge:1, maximumValue:100, minWidth:30});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Style:", minWidth:55});
					var mySelectedStrokeStyle = dropdowns.add({selectedIndex:myStyleId, stringList:myStrokeStylesList, minWidth:200});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Color:", minWidth:55});
					var mySelectedSwatch = dropdowns.add({selectedIndex:myColorId, stringList:mySwatchesList, minWidth:200});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Tint:", minWidth:55});
					var mySelectedTint = percentComboboxes.add({editValue:myTintSave, stringList:myPercentList, smallNudge:5, largeNudge:10, maximumValue:100});
					staticTexts.add({staticLabel:"Stroke:", minWidth:50});
					var mySelectedStroke = measurementComboboxes.add({editValue:myStrokeSave, stringList:mySizeList, smallNudge:0.25, largeNudge:1, maximumValue:100});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Fill color:", minWidth:55});
					var mySelectedFillColor = dropdowns.add({selectedIndex:myFillId, stringList:mySwatchesList, minWidth:200});
				}
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Fill tint:", minWidth:55});
					var mySelectedFillTint = percentComboboxes.add({editValue:myFillTintSave, stringList:myPercentList, smallNudge:5, largeNudge:10, maximumValue:100});
				}				
				with (dialogRows.add()) {
					staticTexts.add({staticLabel:"Align:", minWidth:55});
					var mySelectedAlign = dropdowns.add({selectedIndex:myAlignSave, stringList:myAlignList, minWidth:30});
				}				
			}
			with (dialogRows.add()) {
				staticTexts.add({staticLabel:"Paragraph Style:"});
				var mySelectedParagraphStyle = dropdowns.add({stringList:myParaStylesList, selectedIndex:myParagraphStyleId});
			}
			with (dialogRows.add()) {
				var mySelectedAtLeast = checkboxControls.add({});
				mySelectedAtLeast.checkedState = myAtLeast;
				staticTexts.add({staticLabel:"Автоматическая высота строк"});
			}
			with (dialogRows.add()) {
				var mySelectedControl = checkboxControls.add({});
				mySelectedControl.checkedState = myControl;
				staticTexts.add({staticLabel:"Убирать пробелы и знаки конца абзаца"});
			}
			with (dialogRows.add()) {
				var mySelectedEnd = checkboxControls.add({});
				mySelectedEnd.checkedState = myEnd;
				staticTexts.add({staticLabel:"Сообщать о завершении работы"});
			}		
		}
		with (dialogRows.add()) {			
			dialogColumns.add().staticTexts.add({staticLabel:"\u00A9 Dmitriy N. Sakharov    sakhar@mail.ru"});
		}
	}
	// Показываем
	var myRes = myDialog.show();
	if (!myRes) {
		myDialog.destroy();
		exit();
	}	
	var mytime0 = new Date(); // Запуск таймера
	if (myRadioButton1.selectedButton == 0){
		myUnits = " mm";
	}
	else {myUnits = " pt";}
	var mySpaceBeforeSave = mySelectedSpaceBefore.editValue;
	var mySpaceAfterSave = mySelectedSpaceAfter.editValue;
	var myStyleSave = myStrokeStyles[mySelectedStrokeStyle.selectedIndex];
	var myColorSave = mySwatches[mySelectedSwatch.selectedIndex];
	var myFillSave = mySwatches[mySelectedFillColor.selectedIndex];
	var myParagraphStyleSave = myDoc.paragraphStyles[mySelectedParagraphStyle.selectedIndex];
	var myTintSave = mySelectedTint.editValue;
	var myFillTintSave = mySelectedFillTint.editValue;
	var myStrokeSave = mySelectedStroke.editValue;
	var myTopSave = mySelectedTop.editValue;
	var myBottomSave = mySelectedBottom.editValue;
	var myLeftSave = mySelectedLeft.editValue;
	var myRightSave = mySelectedRight.editValue;
	var myAlignSave = mySelectedAlign.selectedIndex;
	var myEnd = mySelectedEnd.checkedState;
	var myControl = mySelectedControl.checkedState;
	var myAtLeast = mySelectedAtLeast.checkedState;
	// Анализ выделения
	switch(myRadioButton.selectedButton){
		case 0:
			if(selection.length==0 || selection[0].constructor.name == "InsertionPoint" || selection[0].constructor.name == "TextFrame"){
				var myState=confirm("Внимание!!! Не выделен текст!\nОбработка будет проводится по всему документу.");
				if (!myState) {exit();}
				for (i = 0; i < myDoc.stories.length; i++) {
					var mySelect = myDoc.stories[i];
					myTableMaker();
				}	
			}
			else if (selection[0].constructor.name == "Table") {
				mySelect = myDoc.selection[0];
				myTableMaker();
			}
			else {
				mySelect=myDoc.selection[0];
				if(mySelect.tables.length==0){
					alert("В текущем выделении таблиц нет!");
					exit();
				}
				myTableMaker();
			}
			break;
		case 1:
			for (i = 0; i < myDoc.stories.length; i++) {
					var mySelect = myDoc.stories[i];
					myTableMaker();
				}	
			break;
		case 2:
			if(selection.length!=0){
				try{mySelect=myDoc.selection[0].parentStory;}
				catch(err){	mySelect=myDoc.selection[0].parent.parentStory;}
				myTableMaker();
			}
			else {
				var myState=confirm("Внимание!!! Не отмечена story!\nОбработка будет проводится по всему документу.");
				if (!myState) {exit();}
				for (i = 0; i < myDoc.stories.length; i++) {
					var mySelect = myDoc.stories[i];
					myTableMaker();
				}
			}
			break;			
	}
	findGrepPreferences = NothingEnum.nothing;
	changeGrepPreferences = NothingEnum.nothing;		
	if (myEnd){
		// Время работы скрипта
		var mytime1 = new Date(); 
	    var mymin = Number(mytime1.getMinutes() - mytime0.getMinutes()); 
		var mysec = Number(mytime1.getSeconds() - mytime0.getSeconds());
		if (mysec < 0) {
			mymin--;
			mysec = 60 + mysec;
		}
		var myEndSave = 1;
		alert("Обработка завершена!\n" + "Время работы: " + mymin + " мин " + mysec + " с" + "\n\n(с) Dmitriy N. Sakharov \n sakhar@mail.ru");
	}
	writeFile(myIniFile);
}

// Обработка таблиц
function myTableMaker() {
	//Create a table style and a cell style.
	try{var myTableStyle = myDoc.tableStyles.add({name:"myTableStyle"});}
	catch(err){var myTableStyle = myDoc.tableStyles.item("myTableStyle");}	
	try{var myCellStyle = myDoc.cellStyles.add({name:"myCellStyle"});}
	catch(err){var myCellStyle = myDoc.cellStyles.item("myCellStyle");}
	with(myCellStyle) {			
		// Установка отступов в ячейках таблицы
		topInset = myTopSave + myUnits;
		bottomInset = myBottomSave + myUnits;
		leftInset = myLeftSave + myUnits;
		rightInset = myRightSave + myUnits;
		// Установка толщины линий таблицы
		topEdgeStrokeWeight =  bottomEdgeStrokeWeight = leftEdgeStrokeWeight =  rightEdgeStrokeWeight = diagonalLineStrokeWeight = myStrokeSave + " pt";		
		// Установка цвета линий таблицы
		bottomEdgeStrokeColor = topEdgeStrokeColor = leftEdgeStrokeColor = rightEdgeStrokeColor = myColorSave;			
		// Установка оттенка цвета линий таблицы
		topBorderStrokeTint = bottomBorderStrokeTint = leftBorderStrokeTint = rightBorderStrokeTint = myTintSave;			
		// Установка типа линий таблицы
		bottomEdgeStrokeType = topEdgeStrokeType = leftEdgeStrokeType = rightEdgeStrokeType = myStyleSave;			
		// Установка цвета заливки в ячейках таблицы
		fillColor = myFillSave;
		// Установка оттенка цвета заливки в ячейках таблицы
		fillTint = myFillTintSave;
		// Установка вертикального выравнивания в ячейках таблицы
		switch(myAlignSave){
			case 0:
				verticalJustification = 1953460256;
				break;
			case 1:
				verticalJustification = 1667591796;
				break;
			case 2:
				verticalJustification = 1651471469;
				break;			
			case 3:
				verticalJustification = 1785951334;				
				break;			
		}			
		// Установка стиля текста в ячейках таблицы
		appliedParagraphStyle = myParagraphStyleSave;		
	}
	with(myTableStyle) {	
		// Установка отступов сверху и снизу таблицы	
		spaceBefore = mySpaceBeforeSave + myUnits;
		spaceAfter = mySpaceAfterSave + myUnits;
		bodyRegionCellStyle = myCellStyle;		
	}		
	try{var myTablesCount = mySelect.tables.length;}
	catch(err){var myTablesCount = 1;}
	//Creates a progress bar in the "session" JavaScript engine.
	var myProgressPanel;
	var myMaximumValue = myTablesCount;
	var myProgressBarWidth = 300;
	var myIncrement = myMaximumValue/myProgressBarWidth;
	myProgressPanel = myCreateProgressPanel(myMaximumValue, myProgressBarWidth);
	myProgressPanel.show();	
	for (myCounter = 0; myCounter < myTablesCount; myCounter++) {
		myProgressPanel.myProgressBar.value = myCounter;
		try {var myTable = mySelect.tables.item(myCounter);}
		catch(err) {var myTable = mySelect;}
		// Установка высоты строк таблицы
		if(myAtLeast) {
			myTable.rows.everyItem().autoGrow = true;
			myTable.rows.everyItem().minimumHeight = 0;
			myAtLeastSave = 1;
		}		
		myTable.cells.everyItem().clearCellStyleOverrides();		
		myTable.cells.everyItem().texts[0].clearOverrides(OverrideType.all);
		myTable.appliedTableStyle = myTableStyle;
		myTable.clearTableStyleOverrides();		
		//myTable.texts[0].clearOverrides = 1634495520;	
	}
	myProgressPanel.myProgressBar.value = 0;
	myProgressPanel.hide();
	// Обработка текста в каждой таблице
	if(myControl){	
		myControlSave = 1;		
		// Очищаем параметры поиска/замены		
		app.findGrepPreferences = NothingEnum.nothing;
		app.changeGrepPreferences = NothingEnum.nothing;				
		app.findGrepPreferences.appliedParagraphStyle = myParagraphStyleSave;		
		// Удаление пробелов и знаков конца абзаца в начале ячеек таблицы		
		app.findGrepPreferences.findWhat = "^[\\s\\r]+";				
		mySelect.changeGrep();	
		// Удаление пробелов и знаков конца абзаца в конце ячеек таблицы		
		app.findGrepPreferences.findWhat = "[\\s\\r]+$(?!\\r)";		
		mySelect.changeGrep();			
	}				
}

// Запись файла
function writeFile(myIniFile) {
	myIniFile.open("w");	
	var myString = 	myUnits + "\n" + mySpaceBeforeSave + "\n" + mySpaceAfterSave + "\n" + myStyleSave.name + "\n" + myColorSave.name + "\n" + myFillSave.name + "\n" + myParagraphStyleSave.name + "\n" + myTintSave + "\n" + myFillTintSave + "\n" + myStrokeSave + "\n" + myTopSave + "\n" + myBottomSave + "\n" + myLeftSave + "\n" + myRightSave + "\n" + myAlignSave + "\n" + myEndSave + "\n" + myControlSave + "\n" + myAtLeastSave + "\n";
	myIniFile.write(myString);
	myIniFile.close();
}

// Чтение файла
function readFile(myIniFile) {
	myIniFile.open("r");
	var myArray = new Array();
	var myCounter = 0
	while (!myIniFile.eof) {
		myArray[myCounter] = myIniFile.readln();
		myCounter++;
	}
	myIniFile.close();
	myUnits = String(myArray[0]);
	mySpaceBeforeSave = Number(myArray[1]);
	mySpaceAfterSave = Number(myArray[2]);
	myStyleSave = String(myArray[3]);
	myColorSave = String(myArray[4]);
	myFillSave = String(myArray[5]);
	myParagraphStyleSave = String(myArray[6]);
	myTintSave = Number(myArray[7]);
	myFillTintSave = Number(myArray[8]);
	myStrokeSave = Number(myArray[9]);
	myTopSave = Number(myArray[10]);
	myBottomSave = Number(myArray[11]);
	myLeftSave = Number(myArray[12]);
	myRightSave = Number(myArray[13]);
	myAlignSave = Number(myArray[14]);
	myEnd = Boolean(myArray[15]);
	myControl = Boolean(myArray[16]);
	myAtLeast = Boolean(myArray[17]);
}

// Функция создания ProgressBar
function myCreateProgressPanel(myMaximumValue, myProgressBarWidth){
	myProgressPanel = new Window('window', 'Progress');
	with(myProgressPanel){
		myProgressPanel.myProgressBar = add('progressbar', [12, 12, myProgressBarWidth, 24], 0, myMaximumValue);
	}
	return myProgressPanel;
}